@@ -3,17 +3,17 @@ use std::{path::Path, str};
 use anyhow::Context;
 use maplit::hashmap;
 use memofs::{IoResultExt, Vfs};
-use rbx_dom_weak::RbxValue;
 
 use crate::snapshot::{InstanceContext, InstanceMetadata, InstanceSnapshot};
 
-use super::{
-    dir::snapshot_dir, meta_file::AdjacentMetadata, middleware::SnapshotInstanceResult,
-    util::match_trailing,
-};
+use super::{dir::snapshot_dir, meta_file::AdjacentMetadata, util::match_trailing};
 
 /// Core routine for turning Lua files into snapshots.
-pub fn snapshot_lua(context: &InstanceContext, vfs: &Vfs, path: &Path) -> SnapshotInstanceResult {
+pub fn snapshot_lua(
+    context: &InstanceContext,
+    vfs: &Vfs,
+    path: &Path,
+) -> anyhow::Result<Option<InstanceSnapshot>> {
     let file_name = path.file_name().unwrap().to_string_lossy();
 
     let (class_name, instance_name) = if let Some(name) = match_trailing(&file_name, ".server.lua")
@@ -38,9 +38,7 @@ pub fn snapshot_lua(context: &InstanceContext, vfs: &Vfs, path: &Path) -> Snapsh
         .name(instance_name)
         .class_name(class_name)
         .properties(hashmap! {
-            "Source".to_owned() => RbxValue::String {
-                value: contents_str,
-            },
+            "Source".to_owned() => contents_str.into(),
         })
         .metadata(
             InstanceMetadata::new()
@@ -50,8 +48,8 @@ pub fn snapshot_lua(context: &InstanceContext, vfs: &Vfs, path: &Path) -> Snapsh
         );
 
     if let Some(meta_contents) = vfs.read(&meta_path).with_not_found()? {
-        let mut metadata = AdjacentMetadata::from_slice(&meta_contents, &meta_path)?;
-        metadata.apply_all(&mut snapshot);
+        let mut metadata = AdjacentMetadata::from_slice(&meta_contents, meta_path)?;
+        metadata.apply_all(&mut snapshot)?;
     }
 
     Ok(Some(snapshot))
@@ -66,7 +64,7 @@ pub fn snapshot_lua_init(
     context: &InstanceContext,
     vfs: &Vfs,
     init_path: &Path,
-) -> SnapshotInstanceResult {
+) -> anyhow::Result<Option<InstanceSnapshot>> {
     let folder_path = init_path.parent().unwrap();
     let dir_snapshot = snapshot_dir(context, vfs, folder_path)?.unwrap();
 
@@ -74,8 +72,8 @@ pub fn snapshot_lua_init(
         anyhow::bail!(
             "init.lua, init.server.lua, and init.client.lua can \
              only be used if the instance produced by the containing \
-             directory would be a Folder.\n\n\
-
+             directory would be a Folder.\n\
+             \n\
              The directory {} turned into an instance of class {}.",
             folder_path.display(),
             dir_snapshot.class_name
